#ifndef _fft_param_file
#define _fft_param_file

// Parameter holen fr Phase 2 fr Phi, Fibonacci, elliptische Kurven...
void get_fft_parameter(const double phase2, unsigned int &pg, unsigned int &pms)
{

   // precomputing most common used differences:
   // pre_mul_size ist die Intervallgre mit der pro Polynom gesucht wird

     // Je mehr verschiedene, kleine Primfaktoren enthalten sind, desto
     // weniger teilerfremde Zahlen gibt es im Intervall -> desto geringer
     // sind also die Speicherkosten.
     // Bei unserer fft-Version gilt in etwa, da phi*pre_mul_size ungefhr phase2
     // entsprechen sollte; d.h. das Polynom hat die Grenordnung phi und
     // greift ein Intervall der Gre pre_mul_size ab. Und #phi Funktionswerte lassen
     // sich dann schnell ber multipoint_eval ermitteln.
     // Nun ist die Polynomauswertung allerdings besonders effizient, wenn die
     // Polynome und die Anzahl der Punkte Zweierpotenzen sind.
     // Meiner Ansicht nach drfte es daher vorteilhaft sein, pre_mul_size so zu
     // whlen, da es maximal wird fr ein phi, welches nahe unterhalb der
     // gewnschten Zweierpotenz liegt.
     // Da es nun nicht allzu viele praktikable Zweierpotenzen gibt, mit denen
     // wir arbeiten knnen (bei kleinen Zweierpotenzen ist die improved standard
     // continuation berlegen, bei zu groen Zweierpotenzen ist der Speicherbedarf
     // zu hoch), knnen wir geeignete Werte ber eine Tabelle vorgeben.
     // (Fr die Berechnung der Tabellenwerte siehe "fft_param.txt")
     /* Vorschlge wren 
        Polynomgrad    phi(pre_mul_size)     pre_mul_size             		       abgedeckter Suchraum
	2^8 =     256,        240,	  1050=(2)*(3)*(5)^2*(7),			            134.400
	2^9 =     512,        480,	  2310=(2)*(3)*(5)*(7)*(11),		                    591.360
	2^10=    1024,        960,	  4620=(2)^2*(3)*(5)*(7)*(11),		                  2.365.440
	2^11=    2048,	     1920,	  9240=(2)^3*(3)*(5)*(7)*(11),		                  9.461.760
	2^12=    4096,	     4032,       19110=(2)*(3)*(5)*(7)^2*(13),		                 39.137.280
	2^13=    8192,	     7680,       39270=(2)*(3)*(5)*(7)*(11)*(17),                       160.849.920
	2^14=   16384,      16128,       79170=(2)*(3)*(5)*(7)*(13)*(29),                       648.560.640
	2^15=   32768,      31680,      159390=(2)*(3)^2*(5)*(7)*(11)*(23),                   2.611.445.760
	2^16=   65536,      63360,      330330=(2)*(3)*(5)*(7)*(11)^2*(13),                  10.824.253.440
	2^17=  131072,	   126720,      690690=(2)*(3)*(5)*(7)*(11)*(13)*(23),               45.265.059.840
	2^18=  262144,     253440,     1381380=(2)^2*(3)*(5)*(7)*(11)*(13)*(23),            181.060.239.360
	2^19=  524288,     518400,     2852850=(2)*(3)*(5)^2*(7)*(11)*(13)*(19),            747.857.510.400
	2^20= 1048576,    1036800,     5705700=(2)^2*(3)*(5)^2*(7)*(11)*(13)*(19),        2.991.430.041.600
	2^21= 2097152,    2027520,    11741730=(2)*(3)*(5)*(7)*(11)*(13)*(17)*(23),      12.312.096.276.480
	2^22= 4194304,    4055040,    23483460=(2)^2*(3)*(5)*(7)*(11)*(13)*(17)*(23),    49.248.385.105.920
	2^23= 8388608,    8294400,    48498450=(2)*(3)*(5)^2*(7)*(11)*(13)*(17)*(19),   203.417.242.828.800
	2^24=16777216,   16588800,    96996900=(2)^2*(3)*(5)^2*(7)*(11)*(13)*(17)*(19), 813.668.971.315.200
     */

   // da wir pairing verwenden, verschieben sich die Werte leicht...
   pg=256; pms=2*1050; // Startwerte (default)
   if (phase2 >    591360/2*3) { pg=  512; pms= 2* 2310; }
   if (phase2 >   2365440/2*3) { pg= 1024; pms= 2* 4620; }
   if (phase2 >   9461760/2*3) { pg= 2048; pms= 2* 9240; }
   if (phase2 >  39137280/2*3) { pg= 4096; pms= 2*19110; }
   if (phase2 > 160849920/2*3) { pg= 8192; pms= 2*39270; }
   if (phase2 > 648560640/2*3) { pg=16384; pms= 2*79170; }
   if (phase2 > 2611445760.0/2*3)   { pg=32768; pms=2*159390; }
   if (phase2 > 10824253440.0/2*3)  { pg=65536; pms=2*330330; }

#if defined(I_HAVE_LOT_OF_RAM) || defined(I_HAVE_ENDLESS_RAM)
#warning "phi, fibonacci & ecm are allowed to use large amounts of ram (>1GB)!"
   if (phase2 > 45265059840.0/2*3)  { pg=131072; pms=2*690690; }
   if (phase2 > 181060239360.0/2*3) { pg=262144; pms=2*1381380; }
#endif
#ifdef I_HAVE_ENDLESS_RAM /* more than 2GB!! */
#warning "phi, fibonacci & ecm are allowed to use even more memory!"
   if (phase2 > 747857510400.0/2*3) { pg=524288; pms=2*2852850; }
   if (phase2 > 2991430041600.0/2*3) { pg=1048576; pms=2*5705700; }
#endif

   /* Am Rande:
      Bei 1 GB main memory und pg=262144 geht dem Rechner bereits der
      Speicher aus. (Der Speicherbedarf hngt natrlich auch von der Gre
      der zu faktorisierenden Zahl ab.) Wenn der Rechner nur im Hintergrund
      faktorisieren soll und Memory geswapt wird, ist das kontraproduktiv.
      -> Die Werte sind also individuell zu tunen.

      Ideal wre ein Mechanismus, der den Polynomgrad anhand der
      Gre der zu faktorisierenden Zahl und dem maximal zugelassenen
      Speicherbedarf abschtzt. -> ist aber noch nicht implementiert.
   */

}

#endif
